/******************************
    .3ds file chunk reader
	        v1.0

	   ice development
	    www.ice-d.com
	    coded by slip

     ice development 2005

      Refer to chunk.h
******************************/
#include "chunk.h"
#include "types.h"
#include <fstream>

//Returns a pointer to the Next Chunk.
//If the next chunk is not known it is found
Chunk3DS* Chunk3DS::NextChunk(ifstream * afile)
{
	if(!pNextChunk)			//If we don't already have the next chunk
							//If we do we can return the next chunk
	{
		if(GetNext()==0)	//If zero end?
			return(0);

		Chunk3DS * ptempChunk;
		u16 rid=0;
		u32 rnext=0;

		afile->seekg(offset+GetNext());	//Move to next Chunk start
		ptempChunk=new Chunk3DS();		//Create a new Chunk Object

		afile->read((char*)&rid,2);		//Read the ID
		afile->read((char*)&rnext,4);	//Read the length till next

		ptempChunk->SetID(rid);			//Set the ID
		ptempChunk->SetNext(rnext);		//Set the length till Next
		ptempChunk->SetOffset(offset+GetNext());	//Set the offset of the chunk
		ptempChunk->SetLastChunk(this);

		pNextChunk=ptempChunk;
	}

	return (pNextChunk);
};

//Returns a pointer to the Next Sub Chunk.
//If the next chunk is not known it is found
//The difference between a sub chunk and a chunk
//is a subchunk exists within a chunk's data.
//The only difference between this Function
//and NextChunk() is the seek to before the read.
//
//Basically NextChunk finds the Next chunk based
//on the length until next data, and NextSubChunk
//Gets the next chunk directly after the current
//chunk - too bad if its data this function wont
//check for a bogas value for the next chunk.
Chunk3DS* Chunk3DS::NextSubChunk(ifstream * afile, u32 skipBytes)
{
	if(!pNextChunk)			//If we don't already have the next chunk
							//If we do we can return the next chunk
	{
		if(GetNext()==0)	//If zero end?
			return(0);

		Chunk3DS * ptempChunk;
		u16 rid=0;
		u32 rnext=0;

		afile->seekg(offset+6+skipBytes);	//Move to next Chunk start
		ptempChunk=new Chunk3DS();		//Create a new Chunk Object

		afile->read((char*)&rid,2);		//Read the ID
		afile->read((char*)&rnext,4);	//Read the length till next

		ptempChunk->SetID(rid);			//Set the ID
		ptempChunk->SetNext(rnext);		//Set the length till Next
		ptempChunk->SetOffset(offset+6+skipBytes);	//Set the offset of the chunk
											//Offset is current chunk offset + current chunk header
		ptempChunk->SetLastChunk(this);
		pNextChunk=ptempChunk;
	}

	return (pNextChunk);
};

Chunk3DS::Chunk3DS(ifstream * afile,bool &error)
{
	if(afile)
	{
		afile->seekg(0);	//Seek to start of file
		afile->read((char*)&id,2);
		afile->read((char*)&next,4);
		error=false;
		pNextChunk=0;
		pLastChunk=0;
		offset=0;
	}else
		error=true;
}

//Recursive - Scary
Chunk3DS* Walk(u32 toid,u8 direction,bool removeWhileWalking,Chunk3DS* chunk)
{
	Chunk3DS *previousChunk=0, *nextChunk=0;

	if(direction==CHUNK_WALK_BACK)
	{
		previousChunk=chunk->GetLastChunk();
		if(removeWhileWalking)
			delete chunk;		//chunk is now invalid - DO NOT TOUCH!
		if(previousChunk->GetID()==toid)
		{
			previousChunk->SetNextChunk(0);
			return(previousChunk);
		}else
			return(Walk(toid,CHUNK_WALK_BACK,removeWhileWalking,previousChunk));
	}else
	if(direction==CHUNK_WALK_FORWARD)//Can't walk forward just yet ;)
	{
		return(0);
	}else
		return(0);
}

