/******************************
    .3ds file chunk reader
            v1.0

	   ice development
	    www.ice-d.com
	    coded by slip

     ice development 2005

A class that reads a .3ds file
and reads individual chunks.
The next chunk can be found and
used in such a way as though all
chunks have been read.

It's up to the implementation
to determine weather or not to
read a subchunk or the next
chunk.

Data members:
	u16 id;					//Chunk ID
	u32 next;				//from beginning of this chunk to next chunk (aka length of chunk)
	u32 offset;             //Offset of chunk
	Chunk3DS * pNextChunk;  //Pointer to Next Chunk
	Chunk3DS * pLastChunk;  //Pointer to Last Chunk

Sample Usage:

	ifstream * inFile= new ifstream("example.3ds" , ios::binary);
	bool error=0;
	Chunk3DS * firstChunk = new Chunk3DS(inFile,error);
	Chunk3DS * currentChunk=0;

	if(error)
	{
		cout << "Error reading File\n";
		return(-1);
	}

	//Display Chunks
	currentChunk=firstChunk;
	while(currentChunk!=0)
	{
		cout << "\n--------------------\nChunk: id=0x" << hex << currentChunk->GetID() << " next=" << dec << currentChunk->GetNext() << " offset=" << currentChunk->GetOffset();

		if(currentChunk->GetID()==0x4d4d)		//0x4d4d is the main chunk.
		{
			currentChunk=currentChunk->NextSubChunk(inFile);	//Get the Next Chunk
		}else
			currentChunk=currentChunk->NextChunk(inFile);	//Get the Next Chunk
	}
	cout <<"\n End ";

*******************************/
#ifndef _CHUNK_H_
#define _CHUNK_H_

#include "types.h"
#include <fstream>
#include <iostream>
using namespace std;

#define CHUNK_WALK_BACK		1
#define CHUNK_WALK_FORWARD	2

class Chunk3DS
{
public:
	//Make and destroy mwahahaha
	Chunk3DS(){ id=0; next=0; offset=0; pNextChunk=0;};	//
	Chunk3DS(ifstream * afile,bool &error);
	~Chunk3DS(){};
	//Get Chunks
	Chunk3DS* NextChunk(ifstream * afile);
	Chunk3DS* NextSubChunk(ifstream * afile, u32 skipBytes);
	//Get private member data
	u16 GetID(){ return (id); };
	u32 GetNext(){ return (next); };
	u32 GetOffset(){ return (offset); };
	Chunk3DS* GetNextChunk(){ return(pNextChunk); };
	Chunk3DS* GetLastChunk(){ return(pLastChunk); };
	//Set private member data
	void SetID(u16 nid){ id=nid; };
	void SetNext(u32 nnext){ next=nnext; };
	void SetOffset(u32 noffset){ offset=noffset; };
	void SetLastChunk(Chunk3DS* chunk){ pLastChunk=chunk; };
	void SetNextChunk(Chunk3DS* chunk){ pNextChunk=chunk; };
private:
	u16 id;
	u32 next;				//How far till next chunk? - not subchunks
	u32 offset;				//Offset from start of file this chunk exists
	Chunk3DS * pNextChunk;
	Chunk3DS * pLastChunk;
};

//Tool functions

Chunk3DS* Walk(u32 toid,u8 direction,bool removeWhileWalking,Chunk3DS* chunk);

#endif
