/******************************
  .3ds file data extractor
	        v1.0

	   ice development
	    www.ice-d.com
	    coded by slip

     ice development 2005

Makes use of the Chunk3DS class

It started off as a 3ds Converter
but instead of converting to to a
custom format, i decided to just
extract the data into seperate
files. So my version is a extractor,
but this code is a template for
extractors and converters.

As it stands - this code only works
for 3ds files with single objects.
If there is more than one then the
vertex list is extended and when
the face data for the second (or later)
object is read then the index values will
be incorrect. Easily fixed a number of
ways though.

Keep track the number of times vertex data
has been found and make a new list for each
as with the face data - and somehow reference
it to the object. Or if you wanted to make all
objects into one, just keep track of the last
total number of vertices after the last read
and add that to the face data read.

Email me if you've got more questions about this.

Because this is released with the source
Do what ever you feel like, I trust
you'll respect original Authors etc :)

Email: slip@ice-d.com
if you want to talk. I'd also be
interested in how people are using this :)

Take it easy
-slip
******************************/
#include <fstream>
#include <iostream>
#include <list>
#include "chunk.h"
#include "chunktypes.h"

#define _DEBUGGING_OFF //Change to ON for debugging  

//Some types
typedef struct _Vertex
{
	float x;
	float y;
	float z;
}Vertex;

typedef struct _Face
{
	u16 v1;
	u16 v2;
	u16 v3;
	u16 vo;
}Face;

typedef struct _MappingCo
{	
	float s;
	float t;
}MappingCo;

using namespace std;

//Lets get into it
int main(int argc , char **argv)
{
	if(argc < 4)	//I know in my save code down further I don't make the all the out files manditory, but here it stops you.
	{
		cout << "Something is up with your Syntax.\n\n3dsconvert infile.3ds outVertexFile outFaceFile outMappingFile\n\nTry that.\n\nGoto http://www.ice-d.com for more help if you\ncan't find what you need in the readme.txt\n";
		return -1;
	}

//--------------------
//				START EXTRACTION CODE
// This code is all the work done for you - unless you want something more
// from the file. The guts of this program loops and finds the relating chunks
// to vertices, faces and mapping data and extracts it into three lists (just below)
//
// You can scroll all the way down until you see a nice note that its the
// end of the extraction code, where after you can use the lists and save your own
// format.
//--------------------
	//Vectors to hold Vertex positions
	list<Vertex> vertices;
	list<Face> faces;
	list<MappingCo> mappingCoordinates;

	//Files
	ifstream * inFile= new ifstream(argv[1] , ios::binary);
	ofstream * outFile= new ofstream(argv[2] , ios::binary);
	
	bool error=0;		//For error checking

	//Create the first Chunk (will be the main file chunk)
	//and check for error
	Chunk3DS * currentChunk = new Chunk3DS(inFile,error);
	Chunk3DS *currentChunkTemp=0;
	if(error)			//Error is set to true within the Chunk3DS constructor if the file isn't valid
	{
		cout << "Error reading File\n";
		return(-1);
	}

	//Ok files are open, first chunk is found
	//Search through chunks in this order
	/*
	by id												defined
		0x4d4d											MAIN3DS
			|->0x3d3d									EDIT3DS
			     |->0x4000								EDIT_OBJECT
			          |->0x4100							OBJ_TRIMESH
                           //Read Vertex data 4110		TRI_VERTEXL
				           //Read Face Data 4120		TRI_FACEL1
				      |<---|
				 |<---|
				 |->0x4000
				   //Until no more 0x4000
		//End Read

	This requires a walk backwards in the list
	to find the chunk with id 0x3d3d after the
	Vertex and Face read. As the walk
	progresses each chunk is deleted from memory.

	*/

	//Make sure the first chunk is the MAIN3DS chunk
	if(!(currentChunk->GetID()==MAIN3DS))
	{//If not then report an error
		cout << "\nFile is not a valid 3ds file. Well one that I'll use";
		return(-1);
	}

	//Display Chunks
	u32 dontGoDeeperThan=0;
	while(currentChunk)
	{
		//I only want to see 3D Edit chunk, main chunk and Objects
		switch(currentChunk->GetID())
		{
			case MAIN3DS:
				cout << "\nMain Chunk found";
				currentChunk=currentChunk->NextSubChunk(inFile,0);	//Get the Next Chunk
			break;
			case EDIT3DS:
				cout << "\nEdit3DS Chunk found";
				currentChunk=currentChunk->NextSubChunk(inFile,0);	//Get the Next Chunk
			break;
			case EDIT_OBJECT:
				cout << "\nObject Found: ";
				//A holder to test later to make sure we don't exit this chunck from a subchunk
				dontGoDeeperThan=currentChunk->GetOffset()+currentChunk->GetNext();
				//Read past the object name
				char tempChar;
				u32 nameLength;
				u16 objectType;
				objectType=0;
				nameLength=0;
				tempChar=1;
				inFile->seekg(currentChunk->GetOffset()+6);
				while(tempChar!=0)	//Prints the name and tracks how long it is
				{
					inFile->read(&tempChar,1);
					cout << tempChar;
					nameLength++;
				}
				
				//Read the next two bytes to see the object's type
				//inFile->read((char*)&objectType,2);
				//if(objectType==OBJ_TRIMESH)			//It's a mesh :)
				//	cout << " OBJ_TRIMESH : 0x" << hex << objectType;
				
				currentChunk=currentChunk->NextSubChunk(inFile,nameLength);	//Get the Next Chunk
			break;
			case OBJ_TRIMESH:
					cout << "\nObject Mesh Found";
				currentChunk=currentChunk->NextSubChunk(inFile,0);	//Get the Next Chunk
			break;
			case TRI_VERTEXL:
				cout << "\nMesh Vertex List Found";
				//Read the file and extract the vertex info
				u16 numberVertices;
				u32 xi,yi,zi;
				bool doneLooking;
				doneLooking=false;
				float xf,yf,zf;
				Vertex currentVertex;

				inFile->read((char*)&numberVertices,2);
				cout << "\nNumber of Vertices: " << numberVertices << "\n";
				for(int i=0;i<numberVertices;i++)
				{
					inFile->read((char*)&xi,4);
					inFile->read((char*)&yi,4);
					inFile->read((char*)&zi,4);

					//Grab the binary data from the u32's and smack it into a float untouched
					*((u32*)(&xf))=xi;
					*((u32*)(&yf))=yi;
					*((u32*)(&zf))=zi;
					currentVertex.x=xf;
					currentVertex.y=yf;
					currentVertex.z=zf;
					vertices.push_back(currentVertex);
#ifdef _DEBUGGING_ON
					cout << i << ": x= " << hex << xi << " y= " << yi << " z= " << zi << "\n";
#endif
				}

				//Now find face data.
				currentChunk=currentChunk->NextChunk(inFile);	//Get the Next Chunk
			break;
			case TRI_FACEL1:
				cout << "\nMesh Face List Found";
				//Read the file and extract the vertex info
				u16 numberFaces;
				u16 v1,v2,v3,faceOptions;
				Face currentFace;

				inFile->read((char*)&numberFaces,2);
				cout << "\nNumber of Faces: " << numberFaces << "\n";
				for(int i=0;i<numberFaces;i++)
				{
					inFile->read((char*)&v1,2);
					inFile->read((char*)&v2,2);
					inFile->read((char*)&v3,2);
					inFile->read((char*)&faceOptions,2);

					currentFace.v1=v1;
					currentFace.v2=v2;
					currentFace.v3=v3;
					currentFace.vo=faceOptions;
					faces.push_back(currentFace);

#ifdef _DEBUGGING_ON
						//cout << i << ": v1= " << v1 << " v2= " << v2 << " v3= " << v3 << "\n";
#endif
				}

				//Walk back through list until EDIT_OBJECT then find next Chunk
				//currentChunk=Walk(EDIT_OBJECT,CHUNK_WALK_BACK,true,currentChunk);
				//currentChunk is now the last EDIT_OBJECT chunk
				currentChunk=currentChunk->NextChunk(inFile);	//Get the Next Chunk
			break;
			case TRI_MAPPING:
				cout << "\nMesh Vertex Mapping List Found";
				//Read the file and extract the vertex info
				u16 numberVerts;
				u32 s,t;
				float sf,tf;
				MappingCo currentMappingCo;

				inFile->read((char*)&numberVerts,2);
				cout << "\nNumber of Vertices: " << numberVerts << "\n";
				for(int i=0;i<numberVerts;i++)
				{
					inFile->read((char*)&s,4);
					inFile->read((char*)&t,4);
					*((u32*)&sf)=s;
					*((u32*)&tf)=t;
					currentMappingCo.s=sf;
					currentMappingCo.t=tf;
					mappingCoordinates.push_back(currentMappingCo);

					cout << i << ": s= " << sf << " t= " << tf << "\n";
#ifdef _DEBUGGING_ON
						//cout << i << ": s= " << sf << " t= " << tf << "\n";
#endif
				}

				//Walk back through list until EDIT_OBJECT then find next Chunk
				//currentChunk=Walk(EDIT_OBJECT,CHUNK_WALK_BACK,true,currentChunk);
				//currentChunk is now the last EDIT_OBJECT chunk
				currentChunk=currentChunk->NextChunk(inFile);	//Get the Next Chunk
			break;
			//mappingCoordinates
			default:
				currentChunkTemp=currentChunk;					//This is for cleanup later - The escape
																//out of this loop occurs when the file
																//can't be read anymore and currentChunk
																//is NULL, so we need to preserve currentChunk
																//for when it's NextChunk is NULL.
				currentChunk=currentChunk->NextChunk(inFile);	//Get the Next Chunk
			break;
		}
		//Debugging
#ifdef _DEBUGGING_ON
		if(currentChunk)
			cout << "\n--------------------\nChunk: id=0x" << hex << currentChunk->GetID() << " next=" << dec << currentChunk->GetNext() << " offset=" << currentChunk->GetOffset();
#endif

	}
//--------------------
//				END EXTRACTION CODE
//--------------------

//From this point you can use the lists vertices,faces and mappingCoordinates
//and save in your own format. Check upstairs for the format of each type

//MY QUICK HACK KIND OF JOB FOR SAVING DATA.

//I hate reading and writing files - I confuse myself all the time with the
//byte order.I want to store the data as though it's in memory, so I access
//a float array just by indexing. But write insisted on changing the byte
//order or something when writing to the file, so lucky me can't use the raw
//data of the file, i would have to convert it, so i convert it before hand :)

//This may be a problem for other people. So it may be in your best interest
//to at least alter (no need to do a full re-write) this code. Just in the body
//of the for loops.

	u32 wx, wy,wz,o1,o2,o3,o4;
	//Use the data we have and write a file(s)
	if(!outFile)
	{
		cout << "Error Opening File to write Vertices\n";
		return (-1);
	}

	//Vertices
	typedef list<Vertex>::const_iterator LI;
	for(LI i=vertices.begin();i!=vertices.end();i++)
	{
		const Vertex &v=*i;
		*((float*)&wx)=(v.x);
		*((float*)&wy)=(v.y);
		*((float*)&wz)=(v.z);
		//cout << "\nX: " << hex << wx;
		o1=wx&0x000000FF; // -\ swap these two
		o2=wx&0x0000FF00; // -/
		o3=wx&0x00FF0000; // -\ swap these two
		o4=wx&0xFF000000; // -/
		wx=((((o1<<8) | (o2>>8))<<16) | (((o3<<8) | (o4>>8))>>16));

		o1=wy&0x000000FF; // -\ swap these two
		o2=wy&0x0000FF00; // -/
		o3=wy&0x00FF0000; // -\ swap these two
		o4=wy&0xFF000000; // -/
		wy=((((o1<<8) | (o2>>8))<<16) | (((o3<<8) | (o4>>8))>>16));

		o1=wz&0x000000FF; // -\ swap these two
		o2=wz&0x0000FF00; // -/
		o3=wz&0x00FF0000; // -\ swap these two
		o4=wz&0xFF000000; // -/
		wz=((((o1<<8) | (o2>>8))<<16) | (((o3<<8) | (o4>>8))>>16));

		outFile->write((char*)&wx,4);
		outFile->write((char*)&wy,4);
		outFile->write((char*)&wz,4);
		/*//s16 convert
		swx=(v.x*5);
		swy=(v.y*5);
		swz=(v.z*5);
		*((s16*)&wx)=swx;
		*((s16*)&wy)=swy;
		*((s16*)&wz)=swz;
		temp=(wx<<8);
		wx=(wx>>8)|temp;
		temp=(wy<<8);
		wy=(wy>>8)|temp;
		temp=(wz<<8);
		wz=(wz>>8)|temp;
		outFile->write((char*)&wx,2);
		outFile->write((char*)&wy,2);
		outFile->write((char*)&wz,2);*/
		//cout << "\nX: " << v.x*10 << " Y: " << v.y*10 << " Z: " << v.z*10;
	}
	outFile->close();

	//Face Data
	//If the faces were meant to be written
	if(argc>3)
	{
		outFile->open(argv[3],ios::binary);
		if(!outFile)
		{
			cout << "Error Opening File to write Faces\n";
			return (-1);
		}

		typedef list<Face>::const_iterator LFI;
		u16 v1,v2,v3,o1u16,o2u16;
		for(LFI i=faces.begin();i!=faces.end();i++)
		{
			const Face &f=*i;
			v1=f.v1;
			v2=f.v2;
			v3=f.v3;
			o1u16=(v1&0x00FF);
			o2u16=(v1&0xFF00);
			v1=((o1u16<<8) | (o2u16>>8));

			o1u16=(v2&0x00FF);
			o2u16=(v2&0xFF00);
			v2=((o1u16<<8) | (o2u16>>8));

			o1u16=(v3&0x00FF);
			o2u16=(v3&0xFF00);
			v3=((o1u16<<8) | (o2u16>>8));

			outFile->write((char*)&v1,2);
			outFile->write((char*)&v2,2);
			outFile->write((char*)&v3,2);
			//cout << "\nV1: " << f.v1 << " V2: " << f.v2 << " V3: " << f.v3;
		}
	}
	outFile->close();

	//Mapping Data
	//If mapping co-ordinate list should be generated
	if(argc>4)
	{
		outFile->open(argv[4],ios::binary);
		if(!outFile)
		{
			cout << "Error Opening File to write Mapping Data\n";
			return (-1);
		}

		typedef list<MappingCo>::const_iterator LMI;
		u32 sm,tm;//,o1,o2,o3,o4; <- already defined ^^
		for(LMI i=mappingCoordinates.begin();i!=mappingCoordinates.end();i++)
		{
			const MappingCo &mc=*i;
			*((float*)&sm)=mc.s;
			*((float*)&tm)=mc.t;
			o1=sm&0x000000FF; // -\ swap these two
			o2=sm&0x0000FF00; // -/
			o3=sm&0x00FF0000; // -\ swap these two
			o4=sm&0xFF000000; // -/
			sm=((((o1<<8) | (o2>>8))<<16) | (((o3<<8) | (o4>>8))>>16));

			o1=tm&0x000000FF; // -\ swap these two
			o2=tm&0x0000FF00; // -/
			o3=tm&0x00FF0000; // -\ swap these two
			o4=tm&0xFF000000; // -/
			tm=((((o1<<8) | (o2>>8))<<16) | (((o3<<8) | (o4>>8))>>16));

			outFile->write((char*)&sm,4);
			outFile->write((char*)&tm,4);
			//cout << "\nV1: " << f.v1 << " V2: " << f.v2 << " V3: " << f.v3;
		}
	}
	
	//Finish up and clean up the chunks
	if(currentChunkTemp)
	{
		//Clean up all our chunks in memory       \del/
		currentChunk=Walk(MAIN3DS,CHUNK_WALK_BACK,true,currentChunkTemp);
		delete currentChunk;
	}
	inFile->close();
	outFile->close();
	cout <<"\n Complete ";
	return(0);
}
